@@ -23,8 +23,8 @@ import logging
 import getpass
 import os
 import platform
-import subprocess
 import stat
+import subprocess
 try:
     import grp
 except ImportError:
@@ -32,15 +32,21 @@ except ImportError:
 
 from abc import ABCMeta, abstractmethod
 from functools import partial
+from time import sleep
+
+from leap.bitmask.config import flags
+from leap.bitmask.config.leapsettings import LeapSettings
 
 from leap.bitmask.config.providerconfig import ProviderConfig
 from leap.bitmask.services.eip.eipconfig import EIPConfig, VPNGatewaySelector
 from leap.bitmask.util import first
+from leap.bitmask.util import get_path_prefix
 from leap.bitmask.util.privilege_policies import LinuxPolicyChecker
 from leap.bitmask.util import privilege_policies
 from leap.common.check import leap_assert, leap_assert_type
 from leap.common.files import which
 
+
 logger = logging.getLogger(__name__)
 
 
@@ -95,15 +101,12 @@ class VPNLauncher(object):
         return []
 
     @abstractmethod
-    def get_vpn_env(self, providerconfig):
+    def get_vpn_env(self):
         """
         Returns a dictionary with the custom env for the platform.
         This is mainly used for setting LD_LIBRARY_PATH to the correct
         path when distributing a standalone client
 
-        :param providerconfig: provider specific configuration
-        :type providerconfig: ProviderConfig
-
         :rtype: dict
         """
         return {}
@@ -221,15 +224,18 @@ def _try_to_launch_agent():
     """
     Tries to launch a polkit daemon.
     """
-    opts = [
-        "/usr/lib/policykit-1-gnome/polkit-gnome-authentication-agent-1",
-        # XXX add kde thing here
-    ]
-    for cmd in opts:
-        try:
-            subprocess.Popen([cmd], shell=True)
-        except:
-            pass
+    env = None
+    if flags.STANDALONE is True:
+        env = {"PYTHONPATH": os.path.abspath('../../../../lib/')}
+    try:
+        # We need to quote the command because subprocess call
+        # will do "sh -c 'foo'", so if we do not quoute it we'll end
+        # up with a invocation to the python interpreter. And that
+        # is bad.
+        subprocess.call(["python -m leap.bitmask.util.polkit_agent"],
+                        shell=True, env=env)
+    except Exception as exc:
+        logger.exception(exc)
 
 
 class LinuxVPNLauncher(VPNLauncher):
@@ -240,8 +246,7 @@ class LinuxVPNLauncher(VPNLauncher):
     PKEXEC_BIN = 'pkexec'
     OPENVPN_BIN = 'openvpn'
     OPENVPN_BIN_PATH = os.path.join(
-        ProviderConfig().get_path_prefix(),
-        "..", "apps", "eip", OPENVPN_BIN)
+        get_path_prefix(), "..", "apps", "eip", OPENVPN_BIN)
 
     SYSTEM_CONFIG = "/etc/leap"
     UP_DOWN_FILE = "resolv-update"
@@ -314,6 +319,7 @@ class LinuxVPNLauncher(VPNLauncher):
         if _is_pkexec_in_system():
             if not _is_auth_agent_running():
                 _try_to_launch_agent()
+                sleep(0.5)
             if _is_auth_agent_running():
                 pkexec_possibilities = which(kls.PKEXEC_BIN)
                 leap_assert(len(pkexec_possibilities) > 0,
@@ -343,7 +349,7 @@ class LinuxVPNLauncher(VPNLauncher):
         abs_path_in_bundle = os.path.join(cwd, rel_path_in_bundle)
         if os.path.isfile(abs_path_in_bundle):
             return abs_path_in_bundle
-        abs_path_in_system = kls.OPENVPN_DOWN_ROOT_FILE
+        abs_path_in_system = kls.OPENVPN_DOWN_ROOT_PATH
         if os.path.isfile(abs_path_in_system):
             return abs_path_in_system
 
@@ -389,10 +395,9 @@ class LinuxVPNLauncher(VPNLauncher):
         leap_assert(socket_port, "We need a socket port!")
 
         kwargs = {}
-        if ProviderConfig.standalone:
+        if flags.STANDALONE:
             kwargs['path_extension'] = os.path.join(
-                providerconfig.get_path_prefix(),
-                "..", "apps", "eip")
+                get_path_prefix(), "..", "apps", "eip")
 
         openvpn_possibilities = which(self.OPENVPN_BIN, **kwargs)
 
@@ -414,14 +419,22 @@ class LinuxVPNLauncher(VPNLauncher):
         if openvpn_verb is not None:
             args += ['--verb', '%d' % (openvpn_verb,)]
 
-        gateway_selector = VPNGatewaySelector(eipconfig)
-        gateways = gateway_selector.get_gateways()
+        gateways = []
+        leap_settings = LeapSettings()
+        domain = providerconfig.get_domain()
+        gateway_conf = leap_settings.get_selected_gateway(domain)
+
+        if gateway_conf == leap_settings.GATEWAY_AUTOMATIC:
+            gateway_selector = VPNGatewaySelector(eipconfig)
+            gateways = gateway_selector.get_gateways()
+        else:
+            gateways = [gateway_conf]
 
         if not gateways:
             logger.error('No gateway was found!')
             raise VPNLauncherException(self.tr('No gateway was found!'))
 
-        logger.debug("Using gateways ips: {}".format(', '.join(gateways)))
+        logger.debug("Using gateways ips: {0}".format(', '.join(gateways)))
 
         for gw in gateways:
             args += ['--remote', gw, '1194', 'udp']
@@ -497,23 +510,17 @@ class LinuxVPNLauncher(VPNLauncher):
 
         return [openvpn] + args
 
-    def get_vpn_env(self, providerconfig):
+    def get_vpn_env(self):
         """
         Returns a dictionary with the custom env for the platform.
         This is mainly used for setting LD_LIBRARY_PATH to the correct
         path when distributing a standalone client
 
-        :param providerconfig: provider specific configuration
-        :type providerconfig: ProviderConfig
-
         :rtype: dict
         """
-        leap_assert(providerconfig, "We need a provider config")
-        leap_assert_type(providerconfig, ProviderConfig)
-
-        return {"LD_LIBRARY_PATH": os.path.join(
-                providerconfig.get_path_prefix(),
-                "..", "lib")}
+        return {
+            "LD_LIBRARY_PATH": os.path.join(get_path_prefix(), "..", "lib")
+        }
 
 
 class DarwinVPNLauncher(VPNLauncher):
@@ -648,10 +655,9 @@ class DarwinVPNLauncher(VPNLauncher):
             raise EIPNoTunKextLoaded
 
         kwargs = {}
-        if ProviderConfig.standalone:
+        if flags.STANDALONE:
             kwargs['path_extension'] = os.path.join(
-                providerconfig.get_path_prefix(),
-                "..", "apps", "eip")
+                get_path_prefix(), "..", "apps", "eip")
 
         openvpn_possibilities = which(
             self.OPENVPN_BIN,
@@ -669,11 +675,22 @@ class DarwinVPNLauncher(VPNLauncher):
         if openvpn_verb is not None:
             args += ['--verb', '%d' % (openvpn_verb,)]
 
-        gateway_selector = VPNGatewaySelector(eipconfig)
-        gateways = gateway_selector.get_gateways()
+        gateways = []
+        leap_settings = LeapSettings()
+        domain = providerconfig.get_domain()
+        gateway_conf = leap_settings.get_selected_gateway(domain)
 
-        logger.debug("Using gateways ips: {gw}".format(
-            gw=', '.join(gateways)))
+        if gateway_conf == leap_settings.GATEWAY_AUTOMATIC:
+            gateway_selector = VPNGatewaySelector(eipconfig)
+            gateways = gateway_selector.get_gateways()
+        else:
+            gateways = [gateway_conf]
+
+        if not gateways:
+            logger.error('No gateway was found!')
+            raise VPNLauncherException(self.tr('No gateway was found!'))
+
+        logger.debug("Using gateways ips: {0}".format(', '.join(gateways)))
 
         for gw in gateways:
             args += ['--remote', gw, '1194', 'udp']
@@ -760,20 +777,17 @@ class DarwinVPNLauncher(VPNLauncher):
 
         return [command] + cmd_args
 
-    def get_vpn_env(self, providerconfig):
+    def get_vpn_env(self):
         """
         Returns a dictionary with the custom env for the platform.
         This is mainly used for setting LD_LIBRARY_PATH to the correct
         path when distributing a standalone client
 
-        :param providerconfig: provider specific configuration
-        :type providerconfig: ProviderConfig
-
         :rtype: dict
         """
-        return {"DYLD_LIBRARY_PATH": os.path.join(
-                providerconfig.get_path_prefix(),
-                "..", "lib")}
+        return {
+            "DYLD_LIBRARY_PATH": os.path.join(get_path_prefix(), "..", "lib")
+        }
 
 
 class WindowsVPNLauncher(VPNLauncher):
@@ -825,7 +839,7 @@ class WindowsVPNLauncher(VPNLauncher):
 
         openvpn_possibilities = which(
             self.OPENVPN_BIN,
-            path_extension=os.path.join(providerconfig.get_path_prefix(),
+            path_extension=os.path.join(get_path_prefix(),
                                         "..", "apps", "eip"))
 
         if len(openvpn_possibilities) == 0:
@@ -841,10 +855,22 @@ class WindowsVPNLauncher(VPNLauncher):
         if openvpn_verb is not None:
             args += ['--verb', '%d' % (openvpn_verb,)]
 
-        gateway_selector = VPNGatewaySelector(eipconfig)
-        gateways = gateway_selector.get_gateways()
+        gateways = []
+        leap_settings = LeapSettings()
+        domain = providerconfig.get_domain()
+        gateway_conf = leap_settings.get_selected_gateway(domain)
 
-        logger.debug("Using gateways ips: {}".format(', '.join(gateways)))
+        if gateway_conf == leap_settings.GATEWAY_AUTOMATIC:
+            gateway_selector = VPNGatewaySelector(eipconfig)
+            gateways = gateway_selector.get_gateways()
+        else:
+            gateways = [gateway_conf]
+
+        if not gateways:
+            logger.error('No gateway was found!')
+            raise VPNLauncherException(self.tr('No gateway was found!'))
+
+        logger.debug("Using gateways ips: {0}".format(', '.join(gateways)))
 
         for gw in gateways:
             args += ['--remote', gw, '1194', 'udp']
@@ -897,15 +923,12 @@ class WindowsVPNLauncher(VPNLauncher):
 
         return [openvpn] + args
 
-    def get_vpn_env(self, providerconfig):
+    def get_vpn_env(self):
         """
         Returns a dictionary with the custom env for the platform.
         This is mainly used for setting LD_LIBRARY_PATH to the correct
         path when distributing a standalone client
 
-        :param providerconfig: provider specific configuration
-        :type providerconfig: ProviderConfig
-
         :rtype: dict
         """
         return {}
