@@ -95,6 +95,7 @@ class VPN(object):
         self._reactor = reactor
         self._qtsigs = VPNSignals()
 
+        # XXX should get it from config.flags
         self._openvpn_verb = kwargs.get(self.OPENVPN_VERB, None)
 
     @property
@@ -536,7 +537,7 @@ class VPNManager(object):
         """
         Return a dict containing the vpn environment to be used.
         """
-        return self._launcher.get_vpn_env(self._providerconfig)
+        return self._launcher.get_vpn_env()
 
     def terminate_openvpn(self, shutdown=False):
         """
